/*****************************************************************************
 *
 * utils.c
 *
 * Library of useful functions for plugins
 *
 * Copyright (c) 2000 Karl DeBisschop (karl@debisschop.net)
 * License: GPL
 *
 * $Revision: 1.7 $
 * $Date: 2005/07/02 20:38:31 $
 ****************************************************************************/

#define NAGIOSPLUG_API_C 1

#include <stdarg.h>
#include <limits.h>
#include <arpa/inet.h>

#include "common.h"
#include "utils.h"

extern void print_usage (void);
extern const char *progname;

/* NULL-terminate these arrays just to be prudent
 * It's a bit faster and a lot more portable to use two separate
 * arrays for this, as we don't need to rely on strcasecmp being available
 */
const char *status_strings[] = {
	"OK", "WARNING", "CRITICAL", "UNKNOWN", "DEPENDENT", NULL
};
const char *lowercase_status_strings[] = {
	"ok", "warning", "critical", "unknown", "dependent", NULL
};
/* plugins may set this themselves, so export it from here
 * and have the plugins import it explicitly as extern */
int timeout_status = STATE_CRITICAL;

#define STRLEN 64
#define TXTBLK 128

/* **************************************************************************
 * max_state(STATE_x, STATE_y)
 * compares STATE_x to  STATE_y and returns result based on the following
 * STATE_UNKNOWN < STATE_OK < STATE_WARNING < STATE_CRITICAL
 *
 * Note that numerically the above does not hold
 ****************************************************************************/

int
max_state (int a, int b)
{
	if (a == STATE_CRITICAL || b == STATE_CRITICAL)
		return STATE_CRITICAL;
	else if (a == STATE_WARNING || b == STATE_WARNING)
		return STATE_WARNING;
	else if (a == STATE_OK || b == STATE_OK)
		return STATE_OK;
	else if (a == STATE_UNKNOWN || b == STATE_UNKNOWN)
		return STATE_UNKNOWN;
	else if (a == STATE_DEPENDENT || b == STATE_DEPENDENT)
		return STATE_DEPENDENT;
	else
		return max (a, b);
}

void
usage(const char *fmt, ...)
{
	va_list ap;

	printf("%s: ", progname);

	va_start(ap, fmt);
	vprintf(fmt, ap);
	va_end(ap);

	exit (STATE_UNKNOWN);
}

const char *
clean_revstring (const char *revstring)
{
	static char plugin_revision[STRLEN];
	char *ret;

	ret = &plugin_revision[0]; /* avoid compiler warnings */

	if (sscanf (revstring,"$Revision: %[0-9.]",plugin_revision) == 1)
		return ret;

	return "N/A";
}

void
print_revision (const char *command_name, const char *revision_string)
{
	char plugin_revision[STRLEN];

	if (sscanf (revision_string, "$Revision: %[0-9.]", plugin_revision) != 1)
		strncpy (plugin_revision, "N/A", STRLEN);
	printf ("%s (%s %s) %s\n",
					command_name, PACKAGE, VERSION, plugin_revision);
}

const char *
state_text (int result)
{
	if(result < 0 || result >= STATE_OOB)
		return "UNKNOWN";

	return status_strings[result];
}

static void vdie(int result, const char *fmt, va_list ap)
	__attribute__((noreturn));

static void vdie(int result, const char *fmt, va_list ap)
{
	vprintf(fmt, ap);
	putchar('\n'); /* force a newline */
	exit (result);
}

void
die (int result, const char *fmt, ...)
{
	va_list ap;

	va_start (ap, fmt);
	vdie(result, fmt, ap);
	va_end (ap);
}

void crash(const char *fmt, ...)
{
	va_list ap;
	va_start(ap, fmt);
	vdie(STATE_UNKNOWN, fmt, ap);
	va_end(ap);
}

void
set_timeout_status(const char *arg)
{
	int i;

	timeout_status = -1;
	for(i = 0; status_strings[i] != 0; i++) {
		if(status_strings[i][0] == arg[0] ||
		   lowercase_status_strings[i][0] == arg[0])
		{
			timeout_status = i;
			return;
		}
	}

	/* check for numerical value */
	errno = 0;
	timeout_status = (int)strtol(arg, NULL, 0);
	if(timeout_status >= STATE_OK && timeout_status < STATE_OOB && !errno)
		return;

	/* utter failure (really stupid user) */
	printf(_("Timeout status must be numerical between %d and %d, or one of OK, WARNING, CRITICAL or UNKNOWN. Case doesn't matter and ambiguity is allowed.\n"),
	       STATE_OK, STATE_OOB - 1);

	exit(STATE_UNKNOWN);
}


void
timeout_alarm_handler (int signo)
{
	/* don't segfault if someone stupid has set this arbitrarily */
	if(timeout_status < 0 || timeout_status >= STATE_OOB)
		timeout_status = STATE_UNKNOWN;

	if (signo == SIGALRM) {
		printf (_("%s - Plugin timed out after %d seconds\n"),
						status_strings[timeout_status], timeout_interval);
		exit (timeout_status);
	}

	/* fallthrough so we print something in case some stupid plugin
	 * contributor has set this function to the catch-all sighandler */
	printf(_("Caught signal %d in timeout_alarm_handler (bad coding).\n"),
		signo);
	exit(STATE_UNKNOWN);
}

int
is_numeric (char *number)
{
	char tmp[1];
	float x;

	if (!number)
		return FALSE;
	else if (sscanf (number, "%f%c", &x, tmp) == 1)
		return TRUE;
	else
		return FALSE;
}

int
is_positive (char *number)
{
	if (is_numeric (number) && atof (number) > 0.0)
		return TRUE;
	else
		return FALSE;
}

int
is_negative (char *number)
{
	if (is_numeric (number) && atof (number) < 0.0)
		return TRUE;
	else
		return FALSE;
}

int
is_nonnegative (char *number)
{
	if (is_numeric (number) && atof (number) >= 0.0)
		return TRUE;
	else
		return FALSE;
}

int
is_percentage (char *number)
{
	int x;
	if (is_numeric (number) && (x = atof (number)) >= 0 && x <= 100)
		return TRUE;
	else
		return FALSE;
}

int
is_integer (char *number)
{
	long int n;

	if (!number || (strspn (number, "-0123456789 ") != strlen (number)))
		return FALSE;

	n = strtol (number, NULL, 10);

	if (errno != ERANGE && n >= INT_MIN && n <= INT_MAX)
		return TRUE;
	else
		return FALSE;
}

int
is_intpos (char *number)
{
	if (is_integer (number) && atoi (number) > 0)
		return TRUE;
	else
		return FALSE;
}

int
is_intneg (char *number)
{
	if (is_integer (number) && atoi (number) < 0)
		return TRUE;
	else
		return FALSE;
}

int
is_intnonneg (char *number)
{
	if (is_integer (number) && atoi (number) >= 0)
		return TRUE;
	else
		return FALSE;
}

int
is_intpercent (char *number)
{
	int i;
	if (is_integer (number) && (i = atoi (number)) >= 0 && i <= 100)
		return TRUE;
	else
		return FALSE;
}

int
is_option (char *str)
{
	if (!str)
		return FALSE;
	else if (strspn (str, "-") == 1 || strspn (str, "-") == 2)
		return TRUE;
	else
		return FALSE;
}



#ifdef NEED_GETTIMEOFDAY
int
gettimeofday (struct timeval *tv, struct timezone *tz)
{
	tv->tv_usec = 0;
	tv->tv_sec = (long) time ((time_t) 0);
}
#endif



double
delta_time (struct timeval tv)
{
	struct timeval now;

	gettimeofday (&now, NULL);
	return ((double)(now.tv_sec - tv.tv_sec) + (double)(now.tv_usec - tv.tv_usec) / (double)1000000);
}



long
deltime (struct timeval tv)
{
	struct timeval now;
	gettimeofday (&now, NULL);
	return (now.tv_sec - tv.tv_sec)*1000000 + now.tv_usec - tv.tv_usec;
}




void
strip (char *buffer)
{
	size_t x;
	int i;

	for (x = strlen (buffer); x >= 1; x--) {
		i = x - 1;
		if (buffer[i] == ' ' ||
				buffer[i] == '\r' || buffer[i] == '\n' || buffer[i] == '\t')
			buffer[i] = '\0';
		else
			break;
	}
	return;
}


/******************************************************************************
 *
 * Print perfdata in a standard format
 *
 ******************************************************************************/

char *perfdata (const char *label,
 long int val,
 const char *uom,
 int warnp,
 long int warn,
 int critp,
 long int crit,
 int minp,
 long int minv,
 int maxp,
 long int maxv)
{
	char *data = NULL;

	if (strpbrk (label, "'= "))
		asprintf (&data, "'%s'=%ld%s;", label, val, uom);
	else
		asprintf (&data, "%s=%ld%s;", label, val, uom);

	if (warnp)
		asprintf (&data, "%s%ld;", data, warn);
	else
		asprintf (&data, "%s;", data);

	if (critp)
		asprintf (&data, "%s%ld;", data, crit);
	else
		asprintf (&data, "%s;", data);

	if (minp)
		asprintf (&data, "%s%ld", data, minv);

	if (maxp)
		asprintf (&data, "%s;%ld", data, maxv);

	return data;
}


char *fperfdata (const char *label,
 double val,
 const char *uom,
 int warnp,
 double warn,
 int critp,
 double crit,
 int minp,
 double minv,
 int maxp,
 double maxv)
{
	char *data = NULL;

	if (strpbrk (label, "'= "))
		asprintf (&data, "'%s'=", label);
	else
		asprintf (&data, "%s=", label);

	asprintf (&data, "%s%f", data, val);
	asprintf (&data, "%s%s;", data, uom);

	if (warnp)
		asprintf (&data, "%s%f", data, warn);

	asprintf (&data, "%s;", data);

	if (critp)
		asprintf (&data, "%s%f", data, crit);

	asprintf (&data, "%s;", data);

	if (minp)
		asprintf (&data, "%s%f", data, minv);

	if (maxp) {
		asprintf (&data, "%s;", data);
		asprintf (&data, "%s%f", data, maxv);
	}

	return data;
}

#if (defined (__SVR4) && defined (__sun))
int vasprintf(char **ret, const char *format, va_list args)
{
        va_list copy;
        va_copy(copy, args);

        /* Make sure it is determinate, despite manuals indicating otherwise */
        *ret = 0;

        int count = vsnprintf(NULL, 0, format, args);
        if (count >= 0) {
                char* buffer = malloc(count + 1);
                if (buffer != NULL) {
                        count = vsnprintf(buffer, count + 1, format, copy);
                        if (count < 0)
                                free(buffer);
                        else
                                *ret = buffer;
                }
        }
        va_end(args);  // Each va_start() or va_copy() needs a va_end()

        return count;
}

int asprintf(char **strp, const char *fmt, ...)
{
        int size;
        va_list args;
        va_start(args, fmt);
        size = vasprintf(strp, fmt, args);
        va_end(args);
        return size;
}
#endif

