/*
 * Header file for nagios plugins utils.c
 *
 * This file should be included in all plugins
 *
 * The purpose of this package is to provide safer alternatives to C
 * functions that might otherwise be vulnerable to hacking. This
 * currently includes a standard suite of validation routines to be sure
 * that an string argument acually converts to its intended type and a
 * suite of string handling routine that do their own memory management
 * in order to resist overflow attacks. In addition, a few functions are
 * provided to standardize version and error reporting across the entire
 * suite of plugins.
 */


#ifndef NP_UTILS_H
#define NP_UTILS_H

/** types **/

#ifndef HAVE_STRUCT_TIMEVAL
/* Generalized timer that will do microseconds if available */
struct timeval {
	long tv_sec;        /* seconds */
	long tv_usec;  /* microseconds */
};
#endif

/* universal variables, so we might as well declare them here */
#ifndef NAGIOSPLUG_API_C
unsigned int timeout_interval = DEFAULT_SOCKET_TIMEOUT;
#else
extern unsigned int timeout_interval;
#endif

RETSIGTYPE timeout_alarm_handler (int);

/** prototypes and function-like macros **/
#define max(a,b) (((a) > (b)) ? (a) : (b))
#define min(a,b) (((a) < (b)) ? (a) : (b))

void support (void);
const char *clean_revstring (const char *);
void print_revision (const char *, const char *);
void die (int, const char *, ...) __attribute__((noreturn,format(printf, 2, 3)));
void crash(const char *fmt, ...) __attribute__((format(printf, 1, 2)));
void set_timeout_status(const char *arg);

/* functions for testing input types (deprecated in about 15 minutes) */
int is_integer (char *);
int is_intpos (char *);
int is_intneg (char *);
int is_intnonneg (char *);
int is_intpercent (char *);
int is_numeric (char *);
int is_positive (char *);
int is_negative (char *);
int is_nonnegative (char *);
int is_percentage (char *);
int is_option (char *);

#ifndef HAVE_GETTIMEOFDAY
int gettimeofday(struct timeval *, struct timezone *);
#endif

double delta_time (struct timeval tv);
long deltime (struct timeval tv);
void strip (char *);
int max_state (int a, int b);
void usage (const char *fmt, ...) __attribute__((__noreturn__,__format__(__printf__,1,2)));

const char *state_text (int);

char *perfdata (const char *,
 long int,
 const char *,
 int,
 long int,
 int,
 long int,
 int,
 long int,
 int,
 long int);

char *fperfdata (const char *,
 double,
 const char *,
 int,
 double,
 int,
 double,
 int,
 double,
 int,
 double);


/* The idea here is that, although not every plugin will use all of these,
 * most will or should.  Therefore, for consistency, these very common
 * options should have only these meanings throughout the overall suite */

#define STD_LONG_OPTS \
{"version",no_argument,0,'V'},\
{"verbose",no_argument,0,'v'},\
{"help",no_argument,0,'h'},\
{"timeout",required_argument,0,'t'},\
{"critical",required_argument,0,'c'},\
{"warning",required_argument,0,'w'},\
{"hostname",required_argument,0,'H'}

#define COPYRIGHT "Copyright (c) %s Nagios Plugin Development Team\n\
\t<%s>\n\n"

#define UT_HELP_VRSN "\
\nOptions:\n\
 -h, --help\n\
    Print detailed help screen\n\
 -V, --version\n\
    Print version information\n"

#define UT_HOST_PORT "\
 -H, --hostname=ADDRESS\n\
    Host name or IP Address\n\
 -%c, --port=INTEGER\n\
    Port number (default: %s)\n"

#define UT_IPv46 "\
 -4, --use-ipv4\n\
    Use IPv4 connection\n\
 -6, --use-ipv6\n\
    Use IPv6 connection\n"

#define UT_VERBOSE "\
 -v, --verbose\n\
    Show details for command-line debugging (Nagios may truncate output)\n"

#define UT_WARN_CRIT "\
 -w, --warning=DOUBLE\n\
    Response time to result in warning status (seconds)\n\
 -c, --critical=DOUBLE\n\
    Response time to result in critical status (seconds)\n"

#define UT_TIMEOUT "\
 -t, --timeout=INTEGER\n\
    Seconds before connection times out (default: %d)\n"

#define UT_SUPPORT "\n\
Send email to support@op5.se if you have questions regarding use of this\n\
software, suggestions for improvements or patches.\n"

#if (defined (__SVR4) && defined (__sun))
int vasprintf(char **ret, const char *format, va_list args);
int asprintf(char **strp, const char *fmt, ...);
#endif

#endif /* NP_UTILS_H */
